#!/bin/bash
set -xuv
# ./preprocess.sh exp yearb yeare
#
# -- Author: F. Massonnet 
# -- Date  : 20 Jan 2015
# -- Place : IC3, Barcelona
# -- Adapted: November 2017, UCL
#
# -- Purpose: Prepares EC-Earth output to comply with ECmean standard metrics format
#             Works for MMA* and MMO* types of outputs only
#             So far, only verified for monthly mean outputs starting Jan. the 1st and 12 month chunks
#
# -- To do  : Change units of stresses and fluxes

set -o nounset  # Prevents use of unset variables
set -o errexit  # Prevents snowballing

# -------------
# Dry run
if [[ $# -ne 4 ]]; then
  echo "preprocess.sh exp MEMBER yearb yeare"
  echo "Example:"
  echo "preprocess.sh m069 fc1 1854 1858"
  exit
fi
#
#-------------

# ------------------
# Retrieve arguments
exp=$1
MEMBER=$2
yearb=$3
yeare=$4
CHUNK=000
#testing purposes
#exp=a3pe
#MEMBER=fc00
#yearb=1950
#yeare=1969
#year=1950
# ------------------

# ----------------------------
# Load config file from ECmean
. ./config.sh
# ----------------------------

for year in `seq $yearb $yeare`; do
  echo "============== Processing year $year"
  TMPDIR=${SCRADIR}/TMP_${exp}_${MEMBER}_${year}_${RANDOM}
  echo "TEMPORARY DIRECTORY: >>>>>>> $TMPDIR <<<<<<<< "
  mkdir -p $TMPDIR

  mkdir -p $SAVEDIR/${MEMBER}/Post_${year}/ 

  CHUNK=$((10#${CHUNK}+1))
  printf -v CHUNK '%03d' $CHUNK

  # PHASE 1/ EXTRACTION OF THE RAW DATA
  # -----------------------------------
  for suffix in O; do
    echo "Untarring MM${suffix} data..."
    # 1.1. Untar the data
    # -------------------
    tarfile="${DATADIR}/${MEMBER}/outputs/MM${suffix}_${exp}_${STARTDATE}_${MEMBER}_${CHUNK}_${year}0101-${year}1231.tar"
    if [[ ! -f "$tarfile" ]]; then
      echo "File $tarfile not found, thus aborting"
      exit
    fi
        
    tar xf $tarfile -C $TMPDIR      # The -C option specifies the directory where to untar

    # 1.2 Dezip
    # ---------
    echo "Unzipping MM${suffix} data..."
    for file in `ls $TMPDIR/*.nc.gz`; do
      echo "Unzipping $file..."
      gunzip $file
    done

  done # for suffix

  # PHASE 2/ SELECTING, RENAMING, OPERATIONS ON THE ORIGINAL DATA
  # -------------------------------------------------------------

  # In the end, there must be one file "${exp}_${STARTDATE}_${MEMBER}_${year}0101-${year}1231_${var}.nc" 
  # where var is each of the elements below
  listout=("sosaline" "sosstsst" "mlotst" "zos")

  for var in ${listout[@]}; do
    echo "Preparing data for variable $var..."
    fileout=${TMPDIR}/${exp}_${STARTDATE}_${MEMBER}_${year}0101-${year}1231_${var}.nc

    # 2.1. Processing
    # ---------------
    case $var in
      sosstsst)
        case $ECVERSION in
          3.1)
            surface_temperature_ocean=sosstsst
            ;;
          3.*)
            surface_temperature_ocean=tos
            ;;
          *)
            echo "EC-Earth version unknown"
            exit
        esac

        filein=$TMPDIR/${exp}_1m_${year}0101_${year}1231_grid_T.nc
        ${cdo} -L -selvar,${surface_temperature_ocean} $filein $fileout
        ;;
      sosaline)
        case $ECVERSION in
          3.1)
            salinity_ocean=vosaline
            ;;
          3.*)
            salinity_ocean=so
            ;;
          *)
            echo "EC-Earth version unknown"
            exit
        esac
        filein=$TMPDIR/${exp}_1m_${year}0101_${year}1231_grid_T.nc
        ${cdo} -L -sellevidx,1 -selvar,${salinity_ocean} $filein $fileout
        ncrename -v ${salinity_ocean},${var} $fileout
        ;;
      mlotst)
        ocean_mixed_layer_thickness=mldr10_3
        filein=$TMPDIR/${exp}_1m_${year}0101_${year}1231_grid_T.nc
        ${cdo} -L -selvar,${ocean_mixed_layer_thickness} $filein $fileout
        ncrename -v ${ocean_mixed_layer_thickness},${var} $fileout
        ;;
      zos)
        filein=$TMPDIR/${exp}_1m_${year}0101_${year}1231_grid_T.nc
        ${cdo} -L -selvar,zos $filein $fileout
        ;;
      *)
        echo "Error: variable not found"
        exit
    esac
    
    # 2.2 Saving the data
    # -------------------
    echo "Saving the data..."
    cp $fileout $SAVEDIR/${MEMBER}/Post_${year}/
  done    # var
done # year 

echo ""
echo ""
echo "============================"
echo "     SUCCESSFUL preprocess.sh SCRIPT      " 
echo "============================"
